@@ -22,32 +22,30 @@ def add_to_tables(db, obj):
     if alpha_model and has_table:
         db_name = db.name
         table_name = obj.__tablename__.upper()
-        TABLES[db_name]["tables"][table_name] = obj
+        TABLES[db_name.upper()]["tables"][table_name] = obj  # TODO: modfiy
         return True
     return False
 
 
-def get_definitions_modules(
-    modules_list: List[ModuleType], log: AlphaLogger
-) -> List[ModuleType]:
-    """[Get database table definitions from parent or children module list]
+def get_table_model(  # TODO: delete
+    tablename: str, modules_list: List[ModuleType], log: AlphaLogger
+) -> ModuleType:
+    if len(TABLES) == 0:
+        get_definitions_modules_and_tables(modules_list, log)
 
-    Args:
-        modules_list (List[ModuleType]): [parent or children module list]
-        log (AlphaLogger): [description]
+    for bind, bind_tables in TABLES.items():
+        if tablename.upper() in bind_tables:
+            return bind_tables[tablename.upper()]
+    return None
 
-    Returns:
-        List[ModuleType]: [description]
-    """
-    log.info("Start getting definition modules")
-    main = None
 
-    modules = []
-    modules_db = {}
+def get_definitions_modules(modules_list: List[ModuleType], log: AlphaLogger):
+    sub_modules = []
 
     loaded_modules = []
     # TODO replace ?
     for module_r in modules_list:
+        print(f"Import module {module_r}")
         module = (
             importlib.import_module(module_r) if type(module_r) == str else module_r
         )
@@ -84,50 +82,70 @@ def get_definitions_modules(
 
                 loaded_modules.append(module_full_name)
                 try:
+                    print(f"Import submodule {module_full_name}")
                     sub_module = importlib.import_module(module_full_name)
                 except Exception as ex:
                     log.error(f"Cannot load module {module_full_name}", ex=ex)
                     continue
 
-                if not "db" in sub_module.__dict__:
+                """if not "db" in sub_module.__dict__:
                     continue
-                db = sub_module.__dict__["db"]
+                db = sub_module.__dict__["db"]"""
 
-                modules_db[sub_module] = db
+                sub_modules.append(sub_module)
         else:
-            elements = module.__dict__
-            if "alphaz" in module_r and "core" in elements:
-                db = elements["core"].db
-            elif not "db" in elements:
-                LOG.error(
-                    f"Cannot initialise module <{module_r}>, <db> parameter is missing"
+            sub_modules.append(module)
+    return sub_modules
+
+
+def get_definitions_modules_and_tables(
+    modules_list: List[ModuleType], log: AlphaLogger
+) -> List[ModuleType]:
+    """[Get database table definitions from parent or children module list]
+
+    Args:
+        modules_list (List[ModuleType]): [parent or children module list]
+        log (AlphaLogger): [description]
+
+    Returns:
+        List[ModuleType]: [description]
+    """
+    global TABLES
+    log.info("Start getting definition modules")
+
+    modules = []
+    sub_modules = get_definitions_modules(modules_list, log=log)
+
+    for module in sub_modules:
+        is_db_module = False
+        for el in module.__dict__.values():
+            if hasattr(el, "__tablename__"):
+                bind = (
+                    getattr(el, "__bind_key__")
+                    if hasattr(el, "__bind_key__")
+                    else "MAIN"
                 )
-                continue
-            else:
-                db = elements["db"]
-            modules_db[module] = db
+                if not bind in TABLES:
+                    TABLES[bind.upper()] = {}
+                TABLES[bind.upper()][el.__tablename__.upper()] = el
+                is_db_module = True
+        if is_db_module:
+            modules.append(module)
 
-    for module, db in modules_db.items():
-        if db.main and db.name != "main":
-            main = db.name
-        if not db.name in TABLES:
-            tables = db.get_engine().table_names()
-            TABLES[db.name] = {"db": db, "tables": {x.upper(): None for x in tables}}
-
-        for name, obj in module.__dict__.items():
+        """for name, obj in module.__dict__.items():
             if add_to_tables(db, obj):
-                modules.append(module)
+                modules.append(module)"""
 
-    if main and main in TABLES:
-        if not "main" in TABLES:
-            TABLES["main"] = TABLES[main]
+    """if main and main in TABLES:
+        if not CONFIGURATION.MAIN_DATABASE_NAME in TABLES:
+            TABLES[CONFIGURATION.MAIN_DATABASE_NAME] = TABLES[main]
         else:
             for table, obj in TABLES[main]["tables"].items():
-                if not table in TABLES["main"]["tables"]:
-                    TABLES["main"]["tables"][table] = obj
+                if not table in TABLES[CONFIGURATION.MAIN_DATABASE_NAME]["tables"]:
+                    TABLES[CONFIGURATION.MAIN_DATABASE_NAME]["tables"][table] = obj"""
 
     log.info("END getting definition modules")
-    return modules
+    return modules, TABLES
 
 
 class AlphaModelView(ModelView):
