@@ -1,7 +1,7 @@
-
 import functools
 import time
 
+
 def timer(func):
     @functools.wraps(func)
     def wrapper_timer(*args, **kwargs):
@@ -9,6 +9,7 @@ def timer(func):
         value = func(*args, **kwargs)
         toc = time.perf_counter()
         elapsed_time = toc - tic
-        print(f"Elapsed time: {elapsed_time:0.4f} seconds")
+        print(f"Elapsed time: {elapsed_time:0.4f} seconds for {str(func)}")
         return value
+
     return wrapper_timer
