@@ -10,7 +10,11 @@ class ApiAnswer(_base.AlphaDataclass):
     token_status: str = "success"
     status: str = "success"
     error: int = 0
+    warning: int = 0
     status_code: int = 200
     status_description: str = ""
     requester: str = "unknow"
     data: dict = field(default_factory=lambda: {})
+
+    def to_json(self):
+        return self.get_fields_dict()
