@@ -24,9 +24,7 @@ from ...libs import (
     mail_lib,
     flask_lib,
     io_lib,
-    converter_lib,
     os_lib,
-    json_lib,
     config_lib,
     secure_lib,
 )
@@ -42,7 +40,6 @@ from ...utils.time import tic, tac
 from . import _utils, _colorations
 
 from ._route import Route
-from ._parameter import Parameter
 
 
 class AlphaReloaderLoop(ReloaderLoop):
@@ -130,6 +127,8 @@ reloader_loops["alpha"] = AlphaMTimeReloaderLoop
 
 
 class AlphaFlask(Flask):
+    admin_token: str = None
+
     def __init__(
         self,
         *args,
@@ -165,9 +164,37 @@ class AlphaFlask(Flask):
 
         confs = self.conf.get("config")
         if confs is not None:
+            if configuration == "local":
+                local_ban_keys = ["SQLALCHEMY_POOL_TIMEOUT", "SQLALCHEMY_POOL_SIZE"]
+                for local_ban_key in local_ban_keys:
+                    if local_ban_key in confs:
+                        del confs[local_ban_key]
             for key, value in confs.items():
                 self.config[key] = value
 
+    def set_error(
+        self,
+        status: typing.Union[str, AlphaException] = "error",
+        description: str = None,
+        warning: int = 0,
+    ):
+        self.get_current_route().set_error(
+            status=status, description=description, warning=warning
+        )
+
+    def set_status(self, status="success", description=None):
+        self.get_current_route().set_status(status=status, description=description)
+
+    def get_status(self) -> typing.Tuple[str, str]:
+        return self.get_current_route().get_status()
+
+    def set_warning(
+        self,
+        status: typing.Union[str, AlphaException] = "warning",
+        description: str = None,
+    ):
+        self.set_error(status=status, description=description, warning=1)
+
     def set_config(self, name, configuration=None, root=None):
         self.log.info(
             f"Set <{configuration}> configuration for API from {name} in {root}"
@@ -215,32 +242,48 @@ class AlphaFlask(Flask):
                 parameters_values[key] = value
         return parameters_values
 
+    def gets(
+        self,
+        not_none: bool = False,
+        without: List[str] = None,
+        added: Dict[str, object] = None,
+    ) -> Dict[str, object]:
+        return self.get_parameters(**{x: y for x, y in locals().items() if x != "self"})
+
+    def is_error(self) -> bool:
+        return self.get_current_route().is_error()
+
+    def is_warning(self) -> bool:
+        return self.get_current_route().is_warning()
+
     def set_data(self, data):
         self.get_current_route().set_data(data)
 
     def set_file(self, directory, filename):
         self.get_current_route().set_file(directory, filename)
 
-    def get_file(self, directory, filename, attached=False):
-        self.get_current_route().get_file(directory, filename, attached)
+    def get_file(self, directory, filename, as_attachment=True):
+        self.get_current_route().get_file(directory, filename, as_attachment)
 
     def set_html(self, page, parameters={}):
         self.get_current_route().set_html(page, parameters)
 
     def set_databases(self, db_cnx):
-        self.db_cnx = db_cnx
-        if not "main" in db_cnx:
+        self.db_cnx = {x.upper(): y for x, y in db_cnx.items()}
+        if not CONFIGURATION.MAIN_DATABASE_NAME in self.db_cnx:
             self.log.error("You must define a <main> database")
             exit()
 
-        uri = db_cnx["main"]["cnx"]
+        uri = self.db_cnx[CONFIGURATION.MAIN_DATABASE_NAME]["cnx"]
         if ":///" in uri:
             io_lib.ensure_file(uri.split(":///")[1])
-        db_type = db_cnx["main"]["type"]
+        db_type = self.db_cnx[CONFIGURATION.MAIN_DATABASE_NAME]["type"]
         self.config["SQLALCHEMY_DATABASE_URI"] = uri
 
-        for key, cf_db in db_cnx.items():
-            self.config["SQLALCHEMY_BINDS"] = {x: y["cnx"] for x, y in db_cnx.items()}
+        for key, cf_db in self.db_cnx.items():
+            self.config["SQLALCHEMY_BINDS"] = {
+                x: y["cnx"] for x, y in self.db_cnx.items()
+            }
 
         # self.api.config["MYSQL_DATABASE_CHARSET"]           = "utf8mb4"
         # self.api.config["QLALCHEMY_TRACK_MODIFICATIONS"]    = True
@@ -256,20 +299,19 @@ class AlphaFlask(Flask):
         if routes is not None:
             for route in routes:
                 try:
+                    print(f"Import route {route}")
                     module = importlib.import_module(route)
                 except Exception as ex:
-                    self.log.critical(f"Cannot import routes from {route}",ex=ex)
+                    self.log.critical(f"Cannot import routes from {route}", ex=ex)
         self.cache_dir = self.conf.get("directories/cache")
 
-        if routes is None:
-            self.log.info(f"No routes to init")
-        else:
+        if routes is not None:
             self.log.info(f"Init api with routes: {', '.join(routes)}")
 
         # check request
         # ! freeze - dont know why
-        """db              = core.get_database("main")
-        request_model    = database_lib.get_table("main", "request")
+        """db              = core.get_database(CONFIGURATION.MAIN_DATABASE_NAME)
+        request_model    = database_lib.get_table(CONFIGURATION.MAIN_DATABASE_NAME, "request")
         try:
             self.log.debug("Check <request> table")
             obj = db.exist(request_model)
@@ -409,6 +451,7 @@ class AlphaFlask(Flask):
                     "*Miniconda3*",
                     "*werkzeug*",
                     "*AppData*",
+                    "*.zip*",
                 ],
                 reloader_type=reloader_type,
             )
@@ -436,9 +479,9 @@ class AlphaFlask(Flask):
         conf = self.conf.get(name)
         return conf
 
-    def get_url(self, local=False):
+    def get_url(self, local=False, route: str = ""):
         if local:
-            return f"http://localhost:{self.port}"
+            return f"http://localhost:{self.port}/{route}"
         ssl = self.get_config("ssl")
         pref = "https://" if ssl else "http://"
         return pref + self.get_config("host_public")
@@ -446,11 +489,17 @@ class AlphaFlask(Flask):
     def access_denied(self):
         self.get_current_route().access_denied()
 
-    def get(self, name):
+    def get(self, name: str, default=None):
         route = self.get_current_route()
         if route is None:
             return None
-        return route.get(name)
+        return route.get(name, default)
+
+    def set(self, name: str, value):
+        route = self.get_current_route()
+        if route is None:
+            raise AlphaException(f"Cannot find route to assign value {name=}={value}")
+        return route.set(name, value)
 
     def __getitem__(self, key):
         return self.get(key)
@@ -558,7 +607,7 @@ class AlphaFlask(Flask):
     def get_path(self):
         return self.get_current_route().route
 
-    def check_is_admin(self) -> bool:
+    def is_admin(self, check_logged_user: bool = True) -> bool:
         """Check if user is an admin or not.
 
         Args:
@@ -567,44 +616,109 @@ class AlphaFlask(Flask):
         Returns:
             bool: [description]
         """
-        route = self.get_current_route()
+        # route = self.get_current_route()
 
-        user_data = self.get_logged_user()
-        if user_data is not None:
-            if user_data["role"] >= 9:
-                return True
-            else:
-                self.warning(f"Wrong permission: {user_data} is not an admin")
-
-        admin_password = self.conf.get("admin_password")
-        if self["admin"] and admin_password is not None:
-            if secure_lib.check_magic_code(self["admin"], admin_password):
+        admin_password, admin_parameter = (
+            self.conf.get("admins/password"),
+            self["admin"],
+        )
+        if admin_parameter and admin_password is not None:
+            if secure_lib.check_cry_operation_code(admin_parameter, admin_password):
                 return True
 
         ip = request.remote_addr
-        admins_ips = self.conf.get("admins")
+        admins_ips = self.conf.get("admins/ips")
         if admins_ips and (ip in admins_ips or f"::ffff:{ip}" in admins_ips):
             return True
-        else:
-            self.warning(f"Wrong permission: {ip} is not an admin")
+
+        if check_logged_user:
+            user = self.get_logged_user()
+            if user is not None:
+                if user.role >= 9:
+                    return True
         return False
 
+    def get_su_user(self, su_user_id: str, su_user_name: str):
+        from ...models.user import AlphaUser
+
+        admin_users = self.conf.get("admins/users")
+
+        user = None
+        if self.is_admin(check_logged_user=False) and su_user_id is not None:
+            from ...libs import user_lib  # todo: modify
+
+            if admin_users is not None and su_user_id is not None:
+                u = [x for x in admin_users if ("id" in x and x["id"] == su_user_id)]
+                user = u[0] if len(u) != 0 else None
+                user = AlphaUser(**user)
+            user = (
+                user_lib.get_user_data_by_id(su_user_id, True) if user is None else user
+            )
+
+        if (
+            user is None
+            and self.is_admin(check_logged_user=False)
+            and su_user_name is not None
+        ):
+            from ...libs import user_lib  # todo: modify
+
+            if admin_users is not None and su_user_name is not None:
+                u = [
+                    x
+                    for x in admin_users
+                    if ("username" in x and x["username"] == su_user_name)
+                ]
+                user = u[0] if len(u) != 0 else None
+                user = AlphaUser(**user)
+            user = (
+                user_lib.get_user_data_by_username(su_user_name, True)
+                if user is None
+                else user
+            )
+        return user
+
     def get_logged_user(self, algorithms=["HS256"]):
+        from ...utils.api import ADMIN_USER_ID_PUBLIC, ADMIN_USER_NAME_PUBLIC
+        from ...models.user import AlphaUser
+
         algorithms = list(algorithms)
         secret_key = self.config["JWT_SECRET_KEY"]
 
-        user_data = None
-        token = AlphaRequest.get_token()
+        user = None
+        token = (
+            AlphaRequest.get_token() if self.admin_token is None else self.admin_token
+        )
 
-        if token is not None:
+        su_user_id = self[ADMIN_USER_ID_PUBLIC.name]
+        su_user_name = self[ADMIN_USER_NAME_PUBLIC.name]
+
+        if (
+            self.is_admin(check_logged_user=False)
+            and su_user_id is not None
+            or su_user_name is not None
+        ):
+            user = self.get_su_user(su_user_id, su_user_name)
+        elif token is not None:
             try:
                 user_data_token = jwt.decode(token, secret_key, algorithms=algorithms)
             except:
                 return None
+
             from ...libs import user_lib  # todo: modify
 
-            user_data = user_lib.get_user_data_by_id(user_data_token["id"])
-        return user_data
+            user = user_lib.get_user_data_by_id(user_data_token["id"], True)
+            if user is None and self.admin_token == token:
+                return AlphaUser.map_from_token(user_data_token)  # TODO: modify
+
+        perm_config = self.conf.get("auth/users", default={})
+        if (
+            user is not None
+            and perm_config is not None
+            and user.username in perm_config
+            and "user_permissions" in perm_config[user.username]
+        ):
+            user.permissions.extend(perm_config[user.username]["user_permissions"])
+        return user
 
     def is_get(self):
         return request.method == "GET"
