@@ -1,7 +1,7 @@
 import os, json, inspect, copy, sys, socket, re, platform, getpass
 import numpy as np
 from typing import List, Dict
-
+from collections.abc import Iterable
 from sqlalchemy.orm import scoped_session, sessionmaker
 from sqlalchemy.ext.declarative import declarative_base
 
@@ -12,12 +12,27 @@ from ._utils import *
 from ..main._base import AlphaClass
 from ..main._exception import EXCEPTIONS
 from ..logger import AlphaLogger
+from ..main import AlphaException
 
 from ...libs import converter_lib, sql_lib, io_lib
 
 _CONFIGURATIONS = {}
 
 
+def exec_parameters(d):
+    if isinstance(d, dict):
+        for k, v in d.items():
+            d[k] = exec_parameters(v)
+    elif isinstance(d, list):
+        d = [exec_parameters(u) for u in d]
+    elif type(d) == str:
+        matchs = re.findall(r"\$\{([^\}]+)\}", d)
+        for match in matchs:
+            output = eval(match, {"os": os})
+            d = d.replace("${" + match + "}", output)
+    return d
+
+
 class AlphaConfig(AlphaClass):
     __reserved = ["user", "configuration", "project", "ip", "platform"]
 
@@ -36,16 +51,13 @@ class AlphaConfig(AlphaClass):
         reserved: List[str] = [],
         required: List[str] = [],
         user: str = None,
-        default: str = None
+        default: str = None,
     ):
         if hasattr(self, "tmp"):
             return
-        
+
         if default is not None and os.path.isfile(default):
-            default_config = AlphaConfig(
-                f"default_{name}",
-                filepath=default,
-            )
+            default_config = AlphaConfig(f"default_{name}", filepath=default,)
             data = data if data is not None else {}
             merge_configuration(
                 data, source_configuration=default_config.data, replace=False
@@ -184,13 +196,13 @@ class AlphaConfig(AlphaClass):
 
             self.configuration = (
                 self.configuration
-                if self.configuration
+                if self.configuration is not None
                 else (tmp_configuration if tmp_configuration else default_configuration)
             )
 
             tmp_config = {}
             configuration_names = [
-                x.lower() for x in self.data_origin["configurations"].keys()
+                x.lower() for x in self.data_origin.get("configurations", {}).keys()
             ]
             # configuration_names.sort(key=len)
 
@@ -222,6 +234,8 @@ class AlphaConfig(AlphaClass):
 
         self.data = self._replace_parameters(config)
 
+        self.data = exec_parameters(self.data)
+
     def __configure(self):
         # Paths
         if self.is_path("paths"):
@@ -351,9 +365,8 @@ class AlphaConfig(AlphaClass):
                 elif len(indexs) == 1:
                     value = values[indexs[0]]
                 else:
-                    self.error(
-                        "Too many possible value at the same level are specified for parameter <%s>"
-                        % parameter
+                    raise AlphaException(
+                        f"Too many possible value at the same level are specified for parameter <{parameter}>"
                     )
                     exit()
 
@@ -470,7 +483,13 @@ class AlphaConfig(AlphaClass):
         data_origin = self.data_origin if data_origin is None else data_origin
         tmp = self.tmp if tmp is None else tmp
 
-        is_in = any([parameters[0] in y for y in [data, data_origin, tmp]])
+        is_in = any(
+            [
+                parameters[0] in y
+                for y in [data, data_origin, tmp]
+                if isinstance(y, Iterable)
+            ]
+        )
         if not is_in:
             return None
 
