@@ -6,9 +6,11 @@ import numpy as np
 # from ...libs.oracle_lib import Connection
 
 from pymysql.err import IntegrityError
+from dataclasses import fields
 
 from sqlalchemy import inspect as inspect_sqlalchemy
 from sqlalchemy import update, create_engine, event
+import sqlalchemy
 from sqlalchemy.orm import (
     relationships,
     scoped_session,
@@ -18,7 +20,9 @@ from sqlalchemy.orm import (
     RelationshipProperty,
     ColumnProperty,
 )
+from sqlalchemy import desc, asc
 from sqlalchemy.orm.attributes import InstrumentedAttribute
+from sqlalchemy.orm.relationships import RelationshipProperty
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.sql.expression import or_, and_, all_
 from sqlalchemy.sql.elements import BinaryExpression, Null
@@ -28,6 +32,7 @@ from sqlalchemy.exc import OperationalError
 from sqlalchemy.orm.base import object_mapper
 from sqlalchemy.orm.exc import UnmappedInstanceError
 from collections.abc import Iterable
+from flask_sqlalchemy import DefaultMeta
 
 from time import sleep
 import logging
@@ -35,24 +40,46 @@ from .row import Row
 from .utils import get_schema
 
 from ...models.logger import AlphaLogger
-from ...libs import dict_lib, py_lib
+from ...libs import dict_lib, py_lib, database_lib
 from ...models.main import AlphaException
 from .operators import Operators
 from sqlalchemy.sql.elements import Null
 
-def default_return(results, default=None, columns:dict=None, page:int=None, full_count:int=None):
+
+def default_return(
+    results,
+    default=None,
+    columns: dict = None,
+    page: int = None,
+    full_count: int = None,
+    first: bool = False,
+    pagination_mode: str = "raw",
+):
     results = results if results is not None else default
-    columns = {} if columns is None else {x if not hasattr(x,"key") else x.key():y for x,y in columns.items()}
-    
+    columns = (
+        {}
+        if columns is None
+        else {x if not hasattr(x, "key") else x.key(): y for x, y in columns.items()}
+    )
+
     if columns is not None and type(columns) == dict:
         if type(results) == dict:
-            results = {x if x not in columns else columns[x]:y for x,y in results.items()}
+            results = {
+                x if x not in columns else columns[x]: y for x, y in results.items()
+            }
         elif type(results) == list and len(results) != 0 and type(results)[0] == dict:
-            results = [{x if x not in columns else columns[x]:y for x,y in r.items()} for r in results]
-    if page is not None:
+            results = [
+                {x if x not in columns else columns[x]: y for x, y in r.items()}
+                for r in results
+            ]
+
+    if first and type(results) == dict and len(results) == 0:
+        return None
+    if page is not None and pagination_mode == "raw":
         return (results, full_count)
     return results
 
+
 def get_conditions_from_dict(values: dict, model=None, optional: bool = False):
     conditions = []
     for key, value in values.items():
@@ -76,22 +103,22 @@ def get_conditions_from_dict(values: dict, model=None, optional: bool = False):
                     if not isinstance(v, Null):
                         conditions.append(key.like(v))
                     else:
-                        conditions.append(key==v)
+                        conditions.append(key == v)
                 elif Operators.NOT_LIKE.equals(k):
                     if not isinstance(v, Null):
                         conditions.append(~key.like(v))
                     else:
-                        conditions.append(key!=v)
+                        conditions.append(key != v)
                 elif Operators.ILIKE.equals(k):
                     if not isinstance(v, Null):
                         conditions.append(key.ilike(v))
                     else:
-                        conditions.append(key==v)
+                        conditions.append(key == v)
                 elif Operators.NOT_ILIKE.equals(k):
                     if not isinstance(v, Null):
                         conditions.append(~key.ilike(v))
                     else:
-                        conditions.append(key!=v)
+                        conditions.append(key != v)
                 elif Operators.BETWEEN.equals(k):
                     if len(v) != 2:
                         continue
@@ -116,10 +143,22 @@ def get_conditions_from_dict(values: dict, model=None, optional: bool = False):
                     conditions.append(key <= v)
                 elif Operators.NOT_IN.equals(k):
                     v = v if isinstance(v, Iterable) else [v]
-                    conditions.append(key.notin_([v.value for v in v] if all([issubclass(type(v), enum.Enum) for v in v]) else v))
+                    conditions.append(
+                        key.notin_(
+                            [v.value for v in v]
+                            if all([issubclass(type(v), enum.Enum) for v in v])
+                            else v
+                        )
+                    )
                 elif Operators.IN.equals(k):
                     v = v if isinstance(v, Iterable) else [v]
-                    conditions.append(key.in_([v.value for v in v] if all([issubclass(type(v), enum.Enum) for v in v]) else v))
+                    conditions.append(
+                        key.in_(
+                            [v.value for v in v]
+                            if all([issubclass(type(v), enum.Enum) for v in v])
+                            else v
+                        )
+                    )
                 elif Operators.HAS.equals(k):
                     v = get_filters(v, None, optional=optional)
                     for condition in v:
@@ -150,7 +189,9 @@ def get_filters(filters, model=None, optional: bool = False):
     conditions = []
     for filter_c in filters:
         if type(filter_c) == dict:
-            conditions_from_dict = get_conditions_from_dict(filter_c, model, optional=optional)
+            conditions_from_dict = get_conditions_from_dict(
+                filter_c, model, optional=optional
+            )
             conditions.extend(conditions_from_dict)
         elif not optional or (
             optional
@@ -185,6 +226,34 @@ def add_own_encoders(conn, cursor, query, *args):
         cursor.connection.encoders[np.int64] = lambda value, encoders: int(value)
 
 
+def apply_jonctions(model, order_by):
+    query = model.query
+    if order_by is None:
+        return query
+    if type(order_by) != list and type(order_by) != tuple:
+        order_by = [order_by]
+    for item in order_by:
+        if type(item) != str:
+            item = item.key
+        if item is None:
+            continue
+        for rel in inspect_sqlalchemy(model).relationships:
+            if "." in item:
+                item = item.split(".")[0]
+            if item in rel.key:
+                to_join = model.__dict__[item]
+                if (
+                    type(to_join) == InstrumentedAttribute
+                    and type(to_join.prop) == RelationshipProperty
+                ):
+                    query = query.outerjoin(to_join)
+                elif type(to_join) == InstrumentedAttribute:
+                    pass  # TODO: do something ?
+                elif type(to_join) == RelationshipProperty:
+                    pass  # TODO: do something ?
+    return query
+
+
 class RetryingQuery(BaseQuery):
 
     __retry_count__ = 3
@@ -216,16 +285,46 @@ class RetryingQuery(BaseQuery):
                     raise
 
 
-def convert_order_by_str(model, order_by: str, order_by_direction: str=None):
-    if type(order_by) == str and order_by_direction is not None:
-        column = model.__dict__[order_by]
+def convert_order_by_str(model, order_by: str, order_by_direction: str = None):
+    if model is None and order_by is not None and order_by_direction is not None:
+        order_by = sqlalchemy.text(f"{order_by} {order_by_direction}")
+    elif type(order_by) == str and order_by_direction is not None:
+        relation = order_by
+        if "." in order_by:
+            order_by_split = order_by.split(".")
+            relation = order_by_split[0]
+            order_by = order_by_split[1]
+
+        column = model.__dict__[relation]
+
+        if isinstance(column.prop, RelationshipProperty):
+            relation = next(
+                iter(
+                    [
+                        rel
+                        for rel in inspect_sqlalchemy(model).relationships
+                        if relation in rel.key
+                    ]
+                ),
+                None,
+            )
+
+            column = relation.mapper.class_.__dict__[order_by]
+
         order_by = (
             column.asc() if "asc" in order_by_direction.lower() else column.desc()
         )
     elif order_by_direction is not None:
-        order_by = (
-            order_by.asc() if "asc" in order_by_direction.lower() else order_by.desc()
-        )
+        if hasattr(order_by, "asc"):
+            order_by = (
+                order_by.asc()
+                if "asc" in order_by_direction.lower()
+                else order_by.desc()
+            )
+        else:
+            order_by = (
+                asc(order_by) if "asc" in order_by_direction.lower() else desc(order_by)
+            )
     return order_by
 
 
@@ -234,7 +333,9 @@ def apply_order_by(query, order_by, order_by_direction, model):
         return query
 
     if type(order_by) != tuple and type(order_by) != list:
-        query = query.order_by(convert_order_by_str(model, order_by, order_by_direction))
+        query = query.order_by(
+            convert_order_by_str(model, order_by, order_by_direction)
+        )
     else:
         if order_by_direction is not None and len(order_by) == len(order_by_direction):
             order_by_list = [
@@ -270,7 +371,7 @@ class AlphaDatabaseCore(SQLAlchemy):
         main: bool = False,
         **kwargs,
     ):
-        self.db_type: str = config["type"]
+        self.db_type: str = config.get("type", "oracle")
         # cnx = config["cnx"]
 
         """if type(cnx) == dict:
@@ -279,10 +380,8 @@ class AlphaDatabaseCore(SQLAlchemy):
         # event.listen(engine, "before_cursor_execute", add_own_encoders)
         # self._engine = engine
 
-        engine_options = config["engine_options"] if "engine_options" in config else {}
-        session_options = (
-            config["session_options"] if "session_options" in config else {}
-        )
+        engine_options = config.get("engine_options", {})
+        session_options = config.get("session_options", {})
         self.autocommit = (
             "autocommit" in session_options and session_options["autocommit"]
         )
@@ -311,6 +410,8 @@ class AlphaDatabaseCore(SQLAlchemy):
         self.error = None
 
         self.query_str = None
+        self.full_count = None
+        self.pagination_mode = config.get("pagination_mode", "raw")
 
     """def get_engine(self, bind=None):
         return self.db.get_engine(bind=self.name)
@@ -327,13 +428,14 @@ class AlphaDatabaseCore(SQLAlchemy):
         Returns:
             [type]: [description]
         """
+        engine = self.get_engine(bind=bind)
         output = False
         query = "SELECT 1"
-        if self.db_type == "oracle":
+        if self.db_type == "oracle" and not "sql" in str(engine):
             query = "SELECT 1 from dual"
 
         try:
-            self.get_engine(bind=bind).execute(query)
+            engine.execute(query)
             if not self.autocommit:
                 self.session.commit()
             output = True
@@ -350,13 +452,16 @@ class AlphaDatabaseCore(SQLAlchemy):
     def _get_filtered_query(
         self,
         model,
+        query=None,
         filters=None,
         optional_filters=None,
         columns=None,
         likes=None,
         sup=None,
     ):
-        query = model.query
+
+        if query is None:
+            query = model.query
 
         if columns is not None:
             if type(columns) != list and type(columns) != typing.List:
@@ -392,15 +497,24 @@ class AlphaDatabase(AlphaDatabaseCore):
         table_model.__table__.drop(self.get_engine())
 
     def truncate(self, table_model, bind=None):
-        self.execute(f"truncate table {table_model.__tablename__}", bind=bind)
-        if self.db_type == "oracle":
-            try:
-                self.execute(
-                    f"ALTER TABLE {table_model.__tablename__} MODIFY(ID Generated as Identity (START WITH 1));"
-                )
-            except:
-                # TODO: modify
-                pass
+        if bind is None:
+            bind = table_model.__bind_key__
+
+        engine = self.get_engine(bind=bind)
+
+        if self.db_type == "oracle" and not "sql" in str(engine):
+            self.execute(f"truncate table {table_model.__tablename__}", bind=bind)
+            if self.db_type == "oracle":
+                try:
+                    self.execute(
+                        f"ALTER TABLE {table_model.__tablename__} MODIFY(ID Generated as Identity (START WITH 1));"
+                    )
+                except:
+                    # TODO: modify
+                    pass
+        else:
+            table_model.query.delete()
+            self.commit()
 
     def execute(self, query, values=None, commit=True, bind: str = None, close=False):
         return self.execute_query(query, values, commit=commit, bind=bind, close=close)
@@ -455,7 +569,14 @@ class AlphaDatabase(AlphaDatabaseCore):
         )
 
     def get_query_results(
-        self, query, values=None, unique=False, log=None, bind=None, close=False
+        self,
+        query,
+        values=None,
+        unique=False,
+        log=None,
+        bind=None,
+        close=False,
+        dataclass=None,
     ):
         session = self.get_engine(self.app, bind)
 
@@ -464,28 +585,30 @@ class AlphaDatabase(AlphaDatabaseCore):
 
         if log is None:
             log = self.log
-        
+
         if values is not None:
             if type(values) == list and len(values) != 0:
                 dict_values = {}
                 for i, val in enumerate(values):
                     if type(val) == dict:
-                        query = query.replace(
-                            ":%s" % list(val.keys())[0], f":p{i}", 1
-                        )
+                        query = query.replace(":%s" % list(val.keys())[0], f":p{i}", 1)
                         dict_values[f"p{i}"] = list(val.values())[0]
                     else:
                         query = query.replace("?", f":p{i}", 1)
                         dict_values[f"p{i}"] = val
                 values = dict_values
-        
+
         try:
-            resultproxy = session.execute(query, values) if values is not None else session.execute(query)
+            resultproxy = (
+                session.execute(query, values)
+                if values is not None
+                else session.execute(query)
+            )
         except Exception as ex:
             if log is not None:
                 log.error(ex)
             raise ex
-        
+
         results = []
         for rowproxy in resultproxy:
             if hasattr(rowproxy, "items"):
@@ -502,7 +625,7 @@ class AlphaDatabase(AlphaDatabaseCore):
                 for value in results
             ]
         self.query_str = get_compiled_query(query).replace("\n", "")
-        
+
         """
         except Exception as err:
             stack = inspect.stack()
@@ -568,6 +691,17 @@ class AlphaDatabase(AlphaDatabaseCore):
             model, parameters=values_update, commit=commit, test=test, update=True
         )
 
+    def values_to_model(self, model, values):
+        if type(values) != dict:
+            self.log.error("<values must be of type <dict>")
+            return None
+        values = {
+            x if not "." in str(x) else str(x).split(".")[-1]: y
+            for x, y in values.items()
+        }
+        obj = model(**values)
+        return obj
+
     def add_or_update(self, obj, parameters=None, commit=True, test=False, update=True):
         return self.add(
             obj, parameters=parameters, commit=commit, test=test, update=True
@@ -599,7 +733,11 @@ class AlphaDatabase(AlphaDatabaseCore):
             obj = model(**parameters)
 
         if type(obj) == list:
-            self.session.add_all(obj)
+            if not update:
+                self.session.add_all(obj)
+            else:
+                for o in obj:
+                    self.session.merge(o)
         else:
             if not update:
                 self.session.add(obj)
@@ -672,9 +810,10 @@ class AlphaDatabase(AlphaDatabaseCore):
         try:
             session.commit()
         except Exception as ex:
-            raise ex
             self.log.error(ex=ex)
             session.rollback()
+            raise ex
+
             valid = False
         finally:
             if close:
@@ -688,9 +827,9 @@ class AlphaDatabase(AlphaDatabaseCore):
             return self.commit(close=close, session=session)
         return True
 
-    def delete(
-        self, model, filters=None, commit: bool = True, close: bool = False
-    ) -> bool:
+    def delete(self, model, filters, commit: bool = True, close: bool = False) -> bool:
+        if filters is None or len(filters) == 0:
+            raise AlphaException("Cannot delete without specifying filters")
         objs = self.select(model, filters=filters, json=False)
         if len(objs) == 0:
             return False
@@ -700,11 +839,24 @@ class AlphaDatabase(AlphaDatabaseCore):
             return self.commit(close=close)
         return True
 
-    def ensure(self, table_name: str, bind=None, drop: bool = False):
+    def get_bind_tablenames(self, bind: str):
         inspector = Inspector.from_engine(self.get_engine(bind=bind))
         tables = inspector.get_table_names()
-        if not table_name.lower() in tables:
-            request_model = core.get_table(self, bind, table_name)
+        return tables
+
+    def get_table(self, schema: str, table: str):
+        registered_classes = [x for x in self.Model._decl_class_registry.values()]
+        registered_models: typing.Dict[str, DefaultMeta] = {
+            x.__tablename__: x for x in registered_classes if isinstance(x, DefaultMeta)
+        }
+
+        if not table in registered_models:
+            raise AlphaException("cannot_find_table", parameters={"table": table})
+        return registered_models[table]
+
+    def ensure(self, table_name: str, bind=None, drop: bool = False):
+        if not table_name.lower() in self.get_bind_tablenames(bind):
+            request_model = self.get_table(bind, table_name)
 
             self.log.info(f"Creating <{table_name}> table in <{bind}> database")
             try:
@@ -718,6 +870,7 @@ class AlphaDatabase(AlphaDatabaseCore):
                     self.log.error(ex)
 
         """
+        
         #if not cls.__tablename__ in cls.metadata.tables:
         #    cls.metadata.create_all()
         # ensure tests
@@ -750,12 +903,12 @@ class AlphaDatabase(AlphaDatabaseCore):
         optional_filters: list = None,
         first: bool = False,
         json: bool = False,
-        distinct=None,
         unique: InstrumentedAttribute = None,
         count: bool = False,
         order_by=None,
         order_by_direction=None,
         group_by=None,
+        distinct=None,
         limit: int = None,
         columns: list | dict = None,
         close=False,
@@ -767,24 +920,21 @@ class AlphaDatabase(AlphaDatabaseCore):
         per_page: int = 100,
         offset: int = None,
         dataclass=None,
-        default=None
+        default=None,
     ):
+        query = apply_jonctions(model, order_by)
+
         # model_name = inspect.getmro(model)[0].__name__
         # if self.db_type == "mysql": self.test(close=False)
         renames_columns = None
         if type(columns) == dict:
             columns = list(columns.keys())
             renames_columns = columns
-            
+
         if columns is not None and (
-            len(columns) == 0 or len([x for x in columns if x != ""]) == 0
+            len(columns) == 0 or len([x for x in columns if str(x) != ""]) == 0
         ):
             columns = None
-        disabled_relationships = disabled_relationships or []
-        disabled_relationships = [
-            x if type(x) is str else (x.key if hasattr(x, "key") else "")
-            for x in disabled_relationships
-        ]
 
         attributes = {}
         for key, col in dict(model.__dict__).items():
@@ -821,25 +971,13 @@ class AlphaDatabase(AlphaDatabaseCore):
             )
 
         query = self._get_filtered_query(
-            model, filters=filters, optional_filters=optional_filters, columns=columns
+            model,
+            query=query,
+            filters=filters,
+            optional_filters=optional_filters,
+            columns=columns,
         )
 
-        if distinct is not None:
-            query = (
-                query.distinct(distinct)
-                if type(distinct) != tuple
-                else query.distinct(*distinct)
-            )
-
-        if group_by is not None:
-            query = (
-                query.group_by(group_by)
-                if type(group_by) != tuple
-                else query.group_by(*group_by)
-            )
-
-        query = apply_order_by(query, order_by, order_by_direction, model)
-
         return self.select_query(
             query,
             model=model,
@@ -848,6 +986,8 @@ class AlphaDatabase(AlphaDatabaseCore):
             unique=unique,
             count=count,
             limit=limit,
+            order_by=order_by,
+            order_by_direction=order_by_direction,
             close=close,
             flush=flush,
             schema=schema,
@@ -855,9 +995,12 @@ class AlphaDatabase(AlphaDatabaseCore):
             disabled_relationships=disabled_relationships,
             page=page,
             per_page=per_page,
+            offset=offset,
             dataclass=dataclass,
             default=default,
-            columns=renames_columns
+            columns=renames_columns,
+            distinct=distinct,
+            group_by=group_by,
         )
 
     def select_query(
@@ -883,9 +1026,27 @@ class AlphaDatabase(AlphaDatabaseCore):
         offset: int = None,
         dataclass=None,
         default=None,
-        columns: dict = None
+        columns: dict = None,
+        distinct=None,
+        group_by=None,
     ):
-        full_count = None
+        if page is not None and per_page is None:
+            per_page = 100
+        if distinct is not None:
+            query = (
+                query.distinct(distinct)
+                if type(distinct) != tuple
+                else query.distinct(*distinct)
+            )
+
+        if group_by is not None:
+            query = (
+                query.group_by(group_by)
+                if type(group_by) != tuple
+                else query.group_by(*group_by)
+            )
+
+        self.full_count = None
         if dataclass is not None:
             json = True
         if filters is not None:
@@ -895,24 +1056,32 @@ class AlphaDatabase(AlphaDatabaseCore):
             optional_filters = get_filters(optional_filters, model=None, optional=True)
             query = query.filter(and_(*optional_filters))
 
-        query = apply_order_by(query, order_by, order_by_direction, model)
-
         if first and limit is None:
             limit = 1
-        
+
         if page is not None:
-            full_count = query.count()
+            self.full_count = query.count()
+            query = apply_order_by(query, order_by, order_by_direction, model)
             query = query.limit(per_page).offset(page * per_page)
         else:
+            query = apply_order_by(query, order_by, order_by_direction, model)
             if limit is not None:
                 query = query.limit(limit)
             if offset is not None:
                 query = query.offset(offset)
-        if count:
-            results = query.count()
-            self.query_str = get_compiled_query(query).replace("\n", "")
-            self.log.debug(self.query_str)
-            return default_return(results, default=default, columns=columns, page=page,full_count=full_count)
+            if count:
+                results = query.count()
+                self.query_str = get_compiled_query(query).replace("\n", "")
+                # self.log.debug(self.query_str)
+                return default_return(
+                    results,
+                    default=default,
+                    columns=columns,
+                    page=page,
+                    full_count=self.full_count,
+                    first=first,
+                    pagination_mode=self.pagination_mode,
+                )
 
         try:
             results = query.all() if not first else query.first()
@@ -930,9 +1099,17 @@ class AlphaDatabase(AlphaDatabaseCore):
             json = True
         if not json:
             self.query_str = get_compiled_query(query).replace("\n", "")
-            self.log.debug(self.query_str, level=2)
-            return default_return(results, default=default, columns=columns, page=page,full_count=full_count)
-        
+            # self.log.debug(self.query_str, level=2)
+            return default_return(
+                results,
+                default=default,
+                columns=columns,
+                page=page,
+                full_count=self.full_count,
+                first=first,
+                pagination_mode=self.pagination_mode,
+            )
+        # json conversion
         results_json = {}
         if schema is None and model is not None:
             schema = get_schema(
@@ -958,31 +1135,65 @@ class AlphaDatabase(AlphaDatabaseCore):
             )
 
         self.query_str = get_compiled_query(query).replace("\n", "")
-        self.log.debug(self.query_str, level=2)
+        # self.log.debug(self.query_str, level=2)
 
         if unique:
             if type(unique) == str:
                 if not first:
-                    return default_return((
-                        []
-                        if len(results_json) == 0
-                        else [x[unique] for x in results_json]
-                    ), default=default, columns=columns, page=page,full_count=full_count)
+                    return default_return(
+                        (
+                            []
+                            if len(results_json) == 0
+                            else [x[unique] for x in results_json]
+                        ),
+                        default=default,
+                        columns=columns,
+                        page=page,
+                        full_count=self.full_count,
+                        first=first,
+                        pagination_mode=self.pagination_mode,
+                    )
                 else:
-                    return default_return((
-                        results_json[unique]
-                    ), default=default, columns=columns, page=page,full_count=full_count)
+                    return default_return(
+                        (results_json[unique]),
+                        default=default,
+                        columns=columns,
+                        page=page,
+                        full_count=self.full_count,
+                        first=first,
+                        pagination_mode=self.pagination_mode,
+                    )
             else:
                 if not first:
-                    return default_return((
-                        []
-                        if len(results_json) == 0
-                        else [x[unique.key] for x in results_json if unique.key in x]
-                    ), default=default, columns=columns, page=page,full_count=full_count)
+                    return default_return(
+                        (
+                            []
+                            if len(results_json) == 0
+                            else [
+                                x[unique.key] for x in results_json if unique.key in x
+                            ]
+                        ),
+                        default=default,
+                        columns=columns,
+                        page=page,
+                        full_count=self.full_count,
+                        first=first,
+                        pagination_mode=self.pagination_mode,
+                    )
                 else:
-                    return default_return((
-                        results_json[unique.key] if unique.key in results_json else None
-                    ), default=default, columns=columns, page=page,full_count=full_count)
+                    return default_return(
+                        (
+                            results_json[unique.key]
+                            if unique.key in results_json
+                            else None
+                        ),
+                        default=default,
+                        columns=columns,
+                        page=page,
+                        full_count=self.full_count,
+                        first=first,
+                        pagination_mode=self.pagination_mode,
+                    )
         """if disabled_relationships and not json:
             if type(results_json) == dict:
                 results_json = model(**results_json)
@@ -990,12 +1201,34 @@ class AlphaDatabase(AlphaDatabaseCore):
                 results_json = [model(**x) for x in results_json]"""
         if dataclass is not None:
             if len(results_json) == 0:
-                return default_return([] if not first else default, default=default, columns=columns, page=page,full_count=full_count)
+                return default_return(
+                    [] if not first else default,
+                    default=default,
+                    columns=columns,
+                    page=page,
+                    full_count=self.full_count,
+                    first=first,
+                    pagination_mode=self.pagination_mode,
+                )
             if not first:
                 results_json = [dataclass.auto_map_from_dict(r) for r in results_json]
             else:
-                results_json = dataclass.auto_map_from_dict(results_json)
-        return default_return(results_json, default=default, columns=columns, page=page,full_count=full_count)
+                if hasattr(dataclass, "auto_map_from_dict"):
+                    results_json = dataclass.auto_map_from_dict(results_json)
+                else:
+                    field_names = [field.name for field in fields(dataclass)]
+                    results_json = dataclass(
+                        **{x: y for x, y in results_json.items() if x in field_names}
+                    )
+        return default_return(
+            results_json,
+            default=default,
+            columns=columns,
+            page=page,
+            full_count=self.full_count,
+            first=first,
+            pagination_mode=self.pagination_mode,
+        )
 
     def update(
         self,
@@ -1052,10 +1285,7 @@ class AlphaDatabase(AlphaDatabaseCore):
                     for key, value in values.items():
                         if not_none and value is None:
                             continue
-                        if type(key) == str:
-                            setattr(row, key, value)
-                        else:
-                            setattr(row, key.key, value)
+                        setattr(row, key if type(key) == str else key.key, value)
                     self.session.merge(row)
             else:
                 query = self._get_filtered_query(model, filters=filters)
@@ -1081,3 +1311,38 @@ class AlphaDatabase(AlphaDatabaseCore):
             elif type(key) == str and hasattr(model, key) and not key in filters:
                 values_update[model.__dict__[key]] = value
         return values_update
+
+    def process_entries(self, bind, table, values: list, headers: list = None):
+        if headers is not None:
+            headers = [
+                x.lower().replace(" ", "_")
+                if hasattr(x, "lower")
+                else str(x).split(".")[1]
+                for x in headers
+            ]
+
+            columns_names = {}
+            for name, col in table.__dict__.items():
+                if hasattr(col, "expression") and hasattr(col.expression, "name"):
+                    columns_names[col.expression.name] = name
+
+            entries = [
+                table(
+                    **{
+                        columns_names[headers[i]]: database_lib.convert_value(value)
+                        for i, value in enumerate(values_list)
+                        if headers[i] in columns_names
+                    }
+                )
+                for values_list in values
+            ]
+        else:
+            entries = values
+
+        # db.session.query(class_instance).delete()
+        # db.session.add_all(entries)
+        session = self.create_scoped_session(options={"bind": bind})
+
+        for entry in entries:
+            session.merge(entry)
+        session.commit()
