@@ -1,13 +1,15 @@
 import datetime
 from alphaz.models.tests import AlphaTest, test
-from core import core
+from core import core, DB
 
 from alphaz.libs import date_lib
 import alphaz.models.database.main_definitions as defs
 from alphaz.models.database.main_definitions import Test
 from alphaz.models.database.operators import Operators
 
-log = core.get_logger("tests")
+from alphaz.models.database.tests import TestChild, TestChilds
+
+LOG = core.get_logger("tests")
 
 
 class Database(AlphaTest):
@@ -29,6 +31,21 @@ class Database(AlphaTest):
         Test.number_.key: 12,
     }
 
+    test_p = {"id": 1, "name_": "test", "test_child_id": 1}
+    test_p_child = {"id": 1, "name_": "test"}
+    test_p_childs_1 = {
+        "id": 1,
+        "name_": "test",
+        "parent_id": 1,
+        "child_parent_id": 1,
+    }
+    test_p_childs_2 = {
+        "id": 2,
+        "name_": "test2",
+        "parent_id": 1,
+        "child_parent_id": 1,
+    }
+
     def __init__(self):
         super().__init__()
 
@@ -36,9 +53,9 @@ class Database(AlphaTest):
         self.connected = self.db.test()
 
         if not self.connected:
-            log.error("not connected")
+            LOG.error("not connected")
             return
-        self.db.truncate(Test)
+        Test.query.delete()
 
     def count(self):
         rows = self.db.select(Test)
@@ -48,10 +65,219 @@ class Database(AlphaTest):
     def connexion(self):
         return self.connected
 
+    @test(save=False)
+    def truncate(self):
+        rows = self.db.select(Test)
+        self.assert_length(rows, 0)
+
+    def init_childs(self, disabled_relationships: list | dict = None) -> dict:
+        Test.query.delete()
+        TestChild.query.delete()
+        TestChilds.query.delete()
+
+        DB.add(Test(**self.test_p))
+        DB.add(TestChild(**self.test_p_child))
+        DB.add(TestChilds(**self.test_p_childs_1))
+        DB.add(TestChilds(**self.test_p_childs_2))
+
+        rows = DB.select(Test, json=True, disabled_relationships=disabled_relationships)
+        return rows
+
+    @test()
+    def childs_compare_fail(self):
+        rows = self.init_childs()
+
+        full_model = {x: y for x, y in self.test_p.items()}
+        full_model["test_child"] = {x: y for x, y in self.test_p_child.items()}
+        full_model["test_child"]["test_childs"] = [
+            self.test_p_childs_2,
+            self.test_p_childs_2,
+        ]
+        full_model["test_childs"] = [self.test_p_childs_1, self.test_p_childs_2]
+
+        full_model["id"] = 3
+
+        self.assert_length(rows, 1)
+        self.assert_not_equal(
+            rows[0],
+            full_model,
+            ignore={
+                "update_date": None,
+                "test_child": {"update_date": None, "test_childs": ["update_date"]},
+                "test_childs": ["update_date"],
+            },
+        )
+
+    @test()
+    def childs_compare(self):
+        rows = self.init_childs()
+
+        full_model = {x: y for x, y in self.test_p.items()}
+        full_model["test_child"] = {x: y for x, y in self.test_p_child.items()}
+        full_model["test_child"]["test_childs"] = [
+            self.test_p_childs_1,
+            self.test_p_childs_2,
+        ]
+        full_model["test_childs"] = [self.test_p_childs_1, self.test_p_childs_2]
+
+        self.assert_length(rows, 1)
+        self.assert_equal(
+            rows[0],
+            full_model,
+            ignore={
+                "update_date": None,
+                "test_child": {"update_date": None, "test_childs": ["update_date"]},
+                "test_childs": ["update_date"],
+            },
+        )
+
+    @test()
+    def no_test_childs_compare(self):
+        rows = self.init_childs(disabled_relationships=[Test.test_childs])
+
+        full_model = {x: y for x, y in self.test_p.items()}
+        full_model["test_child"] = {x: y for x, y in self.test_p_child.items()}
+        full_model["test_child"]["test_childs"] = [
+            self.test_p_childs_1,
+            self.test_p_childs_2,
+        ]
+        # full_model["test_childs"] = [self.test_p_childs_1, self.test_p_childs_2]
+
+        self.assert_length(rows, 1)
+        self.assert_equal(
+            rows[0],
+            full_model,
+            ignore={
+                "update_date": None,
+                "test_child": {"update_date": None, "test_childs": ["update_date"]},
+                "test_childs": ["update_date"],
+            },
+        )
+
+    @test()
+    def no_test_child_compare(self):
+        rows = self.init_childs(disabled_relationships=[Test.test_child])
+
+        full_model = {x: y for x, y in self.test_p.items()}
+        """full_model["test_child"] = {x: y for x, y in self.test_p_child.items()}
+        full_model["test_child"]["test_childs"] = [
+            self.test_p_childs_1,
+            self.test_p_childs_2,
+        ]"""
+        full_model["test_childs"] = [self.test_p_childs_1, self.test_p_childs_2]
+
+        self.assert_length(rows, 1)
+        self.assert_equal(
+            rows[0],
+            full_model,
+            ignore={
+                "update_date": None,
+                "test_child": {"update_date": None, "test_childs": ["update_date"]},
+                "test_childs": ["update_date"],
+            },
+        )
+
+    @test()
+    def no_test_child_test_childs_compare(self):
+        rows = self.init_childs(
+            disabled_relationships={Test.test_child: TestChild.test_childs}
+        )
+
+        full_model = {x: y for x, y in self.test_p.items()}
+        full_model["test_child"] = {x: y for x, y in self.test_p_child.items()}
+        """full_model["test_child"]["test_childs"] = [
+            self.test_p_childs_1,
+            self.test_p_childs_2,
+        ]"""
+        full_model["test_childs"] = [self.test_p_childs_1, self.test_p_childs_2]
+
+        self.assert_length(rows, 1)
+        self.assert_equal(
+            rows[0],
+            full_model,
+            ignore={
+                "update_date": None,
+                "test_child": {"update_date": None, "test_childs": ["update_date"]},
+                "test_childs": ["update_date"],
+            },
+        )
+
+    @test()
+    def no_test_child_test_childs_compare_notation(self):
+        rows = self.init_childs(
+            disabled_relationships=[
+                {Test.test_child: TestChild.test_childs},
+                Test.test_childs,
+            ]
+        )
+
+        full_model = {x: y for x, y in self.test_p.items()}
+        full_model["test_child"] = {x: y for x, y in self.test_p_child.items()}
+        """full_model["test_child"]["test_childs"] = [
+            self.test_p_childs_1,
+            self.test_p_childs_2,
+        ]"""
+        # full_model["test_childs"] = [self.test_p_childs_1, self.test_p_childs_2]
+
+        self.assert_length(rows, 1)
+        self.assert_equal(
+            rows[0],
+            full_model,
+            ignore={
+                "update_date": None,
+                "test_child": {"update_date": None, "test_childs": ["update_date"]},
+                "test_childs": ["update_date"],
+            },
+        )
+
+    @test()
+    def fail_no_test_child_test_childs_compare(self):
+        rows = self.init_childs(
+            disabled_relationships={Test.test_child: Test.test_childs}
+        )
+
+        full_model = {x: y for x, y in self.test_p.items()}
+        full_model["test_child"] = {x: y for x, y in self.test_p_child.items()}
+        full_model["test_child"]["test_childs"] = [
+            self.test_p_childs_1,
+            self.test_p_childs_2,
+        ]
+        full_model["test_childs"] = [self.test_p_childs_1, self.test_p_childs_2]
+
+        self.assert_length(rows, 1)
+        self.assert_equal(
+            rows[0],
+            full_model,
+            ignore={
+                "update_date": None,
+                "test_child": {"update_date": None, "test_childs": ["update_date"]},
+                "test_childs": ["update_date"],
+            },
+        )
+
+    @test()
+    def no_test_child_and_test_childs_compare(self):
+        rows = self.init_childs(
+            disabled_relationships=[Test.test_child, Test.test_childs]
+        )
+
+        full_model = {x: y for x, y in self.test_p.items()}
+
+        self.assert_length(rows, 1)
+        self.assert_equal(
+            rows[0],
+            full_model,
+            ignore={
+                "update_date": None,
+                "test_child": {"update_date": None, "test_childs": ["update_date"]},
+                "test_childs": ["update_date"],
+            },
+        )
+
     @test(save=False)
     def insert(self):
-        self.db.truncate(Test)
-        self.db.add(Test, self.parameters)
+        Test.query.delete()
+        test = self.db.add(Test, self.parameters)
         rows = self.db.select(Test)
         if len(rows) == 0:
             return False
@@ -60,7 +286,7 @@ class Database(AlphaTest):
 
     @test(save=False)
     def insert2(self):
-        self.db.truncate(Test)
+        Test.query.delete()
         self.db.add(
             Test,
             {
@@ -78,8 +304,9 @@ class Database(AlphaTest):
 
     @test(save=False)
     def insert3(self):
-        self.db.truncate(Test)
-        self.db.add(Test(**self.parameters))
+        Test.query.delete()
+        test = Test(**self.parameters)
+        test2 = self.db.add(test)
         rows = self.db.select(Test)
         if len(rows) == 0:
             return False
@@ -88,10 +315,10 @@ class Database(AlphaTest):
 
     @test(save=False)
     def select_dict(self):
-        self.db.truncate(Test)
+        Test.query.delete()
         self.db.add(Test, self.parameters)
         rows = self.db.select(Test, columns=self.columns, json=True)
-        log.info(f"Selected {len(rows)} elements, expecting 1")
+        LOG.info(f"Selected {len(rows)} elements, expecting 1")
         if len(rows) == 0:
             return False
         values = {x: rows[0][x] for x in self.columns}
@@ -105,7 +332,7 @@ class Database(AlphaTest):
 
     @test(save=False)
     def delete(self):
-        self.db.truncate(Test)
+        Test.query.delete()
         self.db.add(Test, self.parameters)
         self.db.delete(Test, filters=[Test.name_ == self.parameters[Test.name_.key]])
         rows = self.db.select(Test)
@@ -113,7 +340,7 @@ class Database(AlphaTest):
 
     @test(save=False)
     def delete2(self):
-        self.db.truncate(Test)
+        Test.query.delete()
         test = Test(**self.parameters)
         self.db.add(test)
         self.db.delete(test)
@@ -122,7 +349,7 @@ class Database(AlphaTest):
 
     @test(save=False)
     def select_unique(self):
-        self.db.truncate(Test)
+        Test.query.delete()
         self.db.add(Test, self.parameters)
         rows = self.db.select(Test, unique=Test.name_.key)
         valid = len(rows) == 1
@@ -132,7 +359,7 @@ class Database(AlphaTest):
 
     @test(save=False)
     def select_unique2(self):
-        self.db.truncate(Test)
+        Test.query.delete()
         self.db.add(Test, self.parameters)
         rows = self.db.select(Test, unique=Test.name_)
         valid = len(rows) == 1
@@ -142,7 +369,7 @@ class Database(AlphaTest):
 
     @test(save=False)
     def add_update(self):
-        self.db.truncate(Test)
+        Test.query.delete()
         self.db.add(Test, self.parameters)
         parameters = {x: y for x, y in self.parameters.items()}
         parameters[Test.number_.key] += 1
@@ -154,7 +381,7 @@ class Database(AlphaTest):
 
     @test(save=False)
     def add_or_update(self):
-        self.db.truncate(Test)
+        Test.query.delete()
         self.db.add(Test, self.parameters)
         parameters = {x: y for x, y in self.parameters.items()}
         parameters[Test.number_.key] += 1
@@ -167,7 +394,7 @@ class Database(AlphaTest):
 
     @test(save=False)
     def add_or_update_multiple(self):
-        self.db.truncate(Test)
+        Test.query.delete()
         tests = [Test(**self.parameters)]
         for i in range(5):
             parameters = {x: y for x, y in self.parameters.items()}
@@ -183,7 +410,7 @@ class Database(AlphaTest):
     @test(save=False)
     def add_or_update_multiple2(self):
         # take the first insert
-        self.db.truncate(Test)
+        Test.query.delete()
         tests = [Test(**self.parameters)]
         for i in range(5):
             parameters = {x: y for x, y in self.parameters.items()}
@@ -197,7 +424,7 @@ class Database(AlphaTest):
 
     @test(save=False)
     def add_or_update_multiple3(self):
-        self.db.truncate(Test)
+        Test.query.delete()
         tests = [Test(**self.parameters)]
         for i in range(5):
             parameters = {x: y for x, y in self.parameters.items()}
@@ -214,7 +441,7 @@ class Database(AlphaTest):
 
     @test()
     def bulk_update(self):
-        self.db.truncate(Test)
+        Test.query.delete()
 
         rows = self.db.select(Test)
         self.assert_is_empty(rows)
@@ -246,10 +473,10 @@ class Database(AlphaTest):
         numbers = self.db.select(Test, unique=Test.number_)
         self.assert_length(numbers, 1)
         self.assert_equal(set(numbers), set([0]))
-        
+
     @test()
     def bulk_update_2(self):
-        self.db.truncate(Test)
+        Test.query.delete()
 
         rows = self.db.select(Test)
         self.assert_is_empty(rows)
@@ -276,7 +503,7 @@ class Database(AlphaTest):
 
     """@test(save=False)
     def upsert(self):
-        self.db.truncate(Test)
+        Test.query.delete()
         test = Test(**self.parameters)
         self.db.add(test)
         parameters = {x:y for x,y in self.parameters.items()}
@@ -288,56 +515,53 @@ class Database(AlphaTest):
 
     @test()
     def filters(self):
-        self.db.truncate(Test)
+        Test.query.delete()
         tests = [Test(**self.parameters)]
         for i in range(5):
             parameters = {x: y for x, y in self.parameters.items()}
             parameters[Test.number_.key] = i
-            parameters[Test.date_.key] = date_lib.str_to_datetime("2020/01/%s 10:02:03"%(i+1))
+            parameters[Test.date_.key] = date_lib.str_to_datetime(
+                "2020/01/%s 10:02:03" % (i + 1)
+            )
             tests.append(Test(**parameters))
         self.db.add_or_update(tests)
-        
-        tests = self.db.select(Test,
-            filters=[
-                {
-                    Test.number_: {
-                        Operators.IN: 1
-                    }
-                },
-            ]
+
+        tests = self.db.select(Test, filters=[{Test.number_: {Operators.IN: 1}},])
+        self.assert_length(tests, 1)
+
+        tests = self.db.select(
+            Test, filters=[{Test.number_: {Operators.IN: [1, 100]}},]
         )
         self.assert_length(tests, 1)
-        
-        tests = self.db.select(Test,
-            filters=[
-                {
-                    Test.number_: {
-                        Operators.IN: [1,100]
-                    }
-                },
-            ]
-        )
-        self.assert_length(tests, 1)
-        
-        tests = self.db.select(Test,
+
+        tests = self.db.select(
+            Test,
             filters=[
                 {
                     Test.date_: {
-                        Operators.SUPERIOR_OR_EQUAL: date_lib.str_to_datetime("01/01/2020 10:02:03"), 
-                        Operators.INFERIOR_OR_EQUAL:  date_lib.str_to_datetime("03/01/2020 10:02:03")
+                        Operators.SUPERIOR_OR_EQUAL: date_lib.str_to_datetime(
+                            "01/01/2020 10:02:03"
+                        ),
+                        Operators.INFERIOR_OR_EQUAL: date_lib.str_to_datetime(
+                            "03/01/2020 10:02:03"
+                        ),
                     },
                 },
-            ]
+            ],
         )
         self.assert_length(tests, 1)
-        
-        tests = self.db.select(Test,
+
+        tests = self.db.select(
+            Test,
             filters=[
                 {
                     Test.date_: {
-                        Operators.BETWEEN_OR_EQUAL: (date_lib.str_to_datetime("01/01/2020 10:02:03"), date_lib.str_to_datetime("03/01/2020 10:02:03"))
+                        Operators.BETWEEN_OR_EQUAL: (
+                            date_lib.str_to_datetime("01/01/2020 10:02:03"),
+                            date_lib.str_to_datetime("03/01/2020 10:02:03"),
+                        )
                     },
                 },
-            ]
+            ],
         )
-        self.assert_length(tests, 1)
\ No newline at end of file
+        self.assert_length(tests, 1)
